@@ -16,6 +16,7 @@ typedef struct MultiStrauss {
   double *gamma;   /* gamma[i,j] = gamma[i+ntypes*j] for i,j = 0... ntypes-1 */
   double *rad;     /* rad[i,j] = rad[j+ntypes*i] for i,j = 0... ntypes-1 */
   double *rad2;    /* squared radii */
+  double  range2;   /* square of interaction range */
   double *loggamma; /* logs of gamma[i,j] */
   double *period;
   int    *hard;     /* hard[i,j] = 1 if gamma[i,j] ~~ 0 */
@@ -32,7 +33,7 @@ Cdata *straussminit(state, model, algo)
      Algor algo;
 {
   int i, j, ntypes, n2, hard;
-  double g, r, r2, logg;
+  double g, r, r2, logg, range2;
   MultiStrauss *multistrauss;
 
   multistrauss = (MultiStrauss *) R_alloc(1, sizeof(MultiStrauss));
@@ -60,6 +61,8 @@ Cdata *straussminit(state, model, algo)
 
   /* ipar will contain n^2 gamma values followed by n^2 values of r */
 
+  range2 = 0.0;
+
   for(i = 0; i < ntypes; i++) {
     for(j = 0; j < ntypes; j++) {
       g = model.ipar[i + j*ntypes];
@@ -72,8 +75,11 @@ Cdata *straussminit(state, model, algo)
       MAT(multistrauss->hard, i, j, ntypes) = hard; 
       MAT(multistrauss->loggamma, i, j, ntypes) = logg;
       MAT(multistrauss->rad2, i, j, ntypes) = r2;
+      if(r2 > range2) range2 = r2;
     }
   }
+  multistrauss->range2 = range2;
+
   /* periodic boundary conditions? */
   multistrauss->period = model.period;
   multistrauss->per    = (model.period[0] > 0.0);
@@ -96,10 +102,14 @@ double straussmcif(prop, state, cdata)
   double *x, *y;
   double u, v, lg;
   double d2, a, cifval;
+  double range2;
+  double *period;
   MultiStrauss *multistrauss;
 
   multistrauss = (MultiStrauss *) cdata;
-
+  range2 = multistrauss->range2;
+  period = multistrauss->period;
+  
   u  = prop.u;
   v  = prop.v;
   mrk = prop.mrk;
@@ -141,42 +151,44 @@ double straussmcif(prop, state, cdata)
   if(multistrauss->per) { /* periodic distance */
     if(ix > 0) {
       for(j=0; j < ix; j++) {
-	mrkj = marks[j];
-	d2 = dist2(u,v,x[j],y[j],multistrauss->period);
-	if(d2 < MAT(multistrauss->rad2, mrk, mrkj, ntypes)) 
-	  MAT(multistrauss->kount, mrk, mrkj, ntypes)++;
+	IF_CLOSE_PERIODIC_D2(u,v,x[j],y[j],period,range2,d2) {
+	  mrkj = marks[j];
+	  if(d2 < MAT(multistrauss->rad2, mrk, mrkj, ntypes)) 
+	    MAT(multistrauss->kount, mrk, mrkj, ntypes)++;
+	}
+	END_IF_CLOSE_PERIODIC_D2
       }
     }
     if(ixp1 < npts) {
       for(j=ixp1; j<npts; j++) {
-	mrkj = marks[j];
-	d2 = dist2(u,v,x[j],y[j],multistrauss->period);
-	if(d2 < MAT(multistrauss->rad2, mrk, mrkj, ntypes)) 
-	  MAT(multistrauss->kount, mrk, mrkj, ntypes)++;
+	IF_CLOSE_PERIODIC_D2(u,v,x[j],y[j],period,range2,d2) {
+	  mrkj = marks[j];
+	  if(d2 < MAT(multistrauss->rad2, mrk, mrkj, ntypes)) 
+	    MAT(multistrauss->kount, mrk, mrkj, ntypes)++;
+	}
+	END_IF_CLOSE_PERIODIC_D2
       }
     }
   }
   else { /* Euclidean distance */
     if(ix > 0) {
       for(j=0; j < ix; j++) {
-	mrkj = marks[j];
-	a = MAT(multistrauss->rad2, mrk, mrkj, ntypes); 
-	a -= pow(u - x[j], 2);
-	if(a > 0) {
-	  a -= pow(v - y[j], 2);
-	  if(a > 0) MAT(multistrauss->kount, mrk, mrkj, ntypes)++;
+        IF_CLOSE_D2(u, v, x[j], y[j], range2, d2) {
+	  mrkj = marks[j];
+	  if(d2 < MAT(multistrauss->rad2, mrk, mrkj, ntypes)) 
+	    MAT(multistrauss->kount, mrk, mrkj, ntypes)++;
 	}
+	END_IF_CLOSE_D2
       }
     }
     if(ixp1 < npts) {
       for(j=ixp1; j<npts; j++) {
-	mrkj = marks[j];
-	a = MAT(multistrauss->rad2, mrk, mrkj, ntypes); 
-	a -= pow(u - x[j], 2);
-	if(a > 0) {
-	  a -= pow(v - y[j], 2);
-	  if(a > 0) MAT(multistrauss->kount, mrk, mrkj, ntypes)++;
+        IF_CLOSE_D2(u, v, x[j], y[j], range2, d2) {
+	  mrkj = marks[j];
+	  if(d2 < MAT(multistrauss->rad2, mrk, mrkj, ntypes)) 
+	    MAT(multistrauss->kount, mrk, mrkj, ntypes)++;
 	}
+	END_IF_CLOSE_D2
       }
     }
   }
